#!/usr/bin/bash

while getopts ":a:r:w:t:p:" opt; do
  case $opt in
    a) 
    # specify tested architecture
    arch="$OPTARG"
    ;;
    r) 
    # specify the rpm absolute path
    rpmdir="$OPTARG"
    ;;
    w) 
    # specify the work space
    workdir="$OPTARG"
    ;;
    t)
    # specify the old or new
    tag_name="$OPTARG"
    ;;
    p)
    # specify the package name (optional)
    package_name="$OPTARG"
    ;;
    \?) 
    echo "Invalid option -$OPTARG" >&2
    exit 1
    ;;
  esac
done


# we need all the argument, so add check on it
if [ -z "$arch" ]; then
  echo "use -a to specify arch"
  exit 1
fi

if [ -z "$rpmdir" ]; then
  echo "use -r to specify taskid"
  exit 1
fi

if [ -z "$workdir" ]; then
  echo "use -w to specify work space"
  exit 1
fi

if [ -z "$tag_name" ]; then
  echo "use -t to specify old or new"
  exit 1
fi
# prepare
dnf install createrepo findutils file jq -y

check_executable(){
# sometime exit 1 doesn't mean failure
    set +e
    success=1
    local result="${1}_result"
    echo "\n========= **Start ${arch} Test.** =========" > $result
    echo "yum repo as follows:\n" >> $result
    ls /etc/yum.repos.d/ >> $result
    echo "============== ** Start check ${1} executable **==================" | tee -a $result
    echo '{}' > ${1}_${arch}_executable_result.json
    while read -r line;
    do
        if echo "$line" | grep -q "test"; then
            echo "Skip test package: $line."
            continue
        fi

        echo "Install $line: "
        dnf install --best $line -y > ${1}_${arch}_single_install_info 2>&1
        if [ $? -ne 0 ]; then
            echo "\`Installed\` $line \`failed\`." | tee -a $result
            echo "no need to test remove. " | tee -a $result
            cat ${1}_${arch}_single_install_info | tee -a $result 
            cat ${1}_${arch}_single_install_info | tee -a ${1}_${arch}_compat_executable_result.log
            echo "===============================\n" | tee -a $result
            success=0
        else
            grep -q "scriptlet failed" ${1}_${arch}_single_install_info
            if [ $? -eq 0 ]; then
                echo "\`Installed\` $line \`success\` with \`scriptlet failed\`." | tee -a $result
                err_info=`grep "scriptlet failed" ${1}_${arch}_single_install_info`
                echo "${err_info}" | tee -a $result
            else
                echo "\`Installed\` $line \`success\`." | tee -a $result
                grep -q "warning:" ${1}_${arch}_single_install_info
                if [ $? -eq 0 ]; then
                    echo "with some \`warnings: \`" | tee -a $result
                    grep "warning:" ${1}_${arch}_single_install_info | tee -a $result
                fi
            fi
            # 安装一个包，开始check_compatibility
            unsupported_types="ELF 64-bit LSB shared object"
            supported_types=("ASCII text executable" "ELF 64-bit LSB")
            args=("--help" "-h" "-v" "--info")
            output=""
            package=$(dnf repoquery --qf "%{NAME}" --installed $line)
            echo "package $package"
            filelist=$(rpm -ql "$package")
            for file in $filelist; do
                if echo "$file" | grep -E "\.debug$|changelog|readme|/usr/share/man|/usr/share/doc|/usr/share/.*-doc|\.html$|.*\.pyc$|/usr/lib/\.build-id/"; then
                    continue
                fi

                file="$file"
                if [ ! -d "$file" ]; then
                    file_type=$(file -b "$file")
                    supported_type=0
                    for t in "${supported_types[@]}"; do
                        if echo "$file_type" | grep -w "$t" | grep -E -v "$unsupported_types"; then
                            supported_type=1
                            break
                        fi
                    done
                    if [ "$supported_type" -eq 1 ] && [ -x "$file" ]; then
                        echo "========================================command $file========================================" | tee -a "${1}_${arch}_compat_executable_result.log"
                        for arg in "${args[@]}"; do
                            argList=$(timeout -s SIGKILL 10s $file $arg 2>&1)
                            exit_status=$?
                            command_key="$file $arg"
                            argList=$(echo "$argList" | tr '"' "'")
                            if [ $exit_status -ne 0 ]; then
                                if [ $exit_status -eq 124 ]; then
                                    echo "**********$command_key timed out, ret=$exit_status, please check and handle it.**********" | tee -a ${1}_${arch}_compat_executable_result.log ${1}_${arch}_compat_executable_failed.log
                                    jq ". | .[\"$file\"] = \"$command_key timed out\"" ${1}_${arch}_executable_result.json > tmp && mv tmp ${1}_${arch}_executable_result.json
                                else
                                    echo "**********$command_key failed, ret=$exit_status**********" | tee -a ${1}_${arch}_compat_executable_result.log ${1}_${arch}_compat_executable_failed.log
                                    # 单个反斜杠替换成两个反斜杠
                                    argList=${argList//\\/\\\\}
                                    jq ". | .[\"$file\"] = \"$argList\"" ${1}_${arch}_executable_result.json > tmp && mv tmp ${1}_${arch}_executable_result.json
                                    echo "$argList" | tee -a ${1}_${arch}_compat_executable_result.log
                                fi
                                continue  # Skip to the next argument
                            fi

                            jq ". | .[\"$file\"] = \"$argList\"" ${1}_${arch}_executable_result.json > tmp && mv tmp ${1}_${arch}_executable_result.json
                            echo "**********$command_key success**********" | tee -a ${1}_${arch}_compat_executable_result.log
                            echo "$argList" | tee -a ${1}_${arch}_compat_executable_result.log
                            break
                        done
                    fi
                fi
            done

            # we dont need protect them in testing environ, which would disturb checking by rejecting to removed.
            rm -f /etc/dnf/protected.d/grub2*
            rm -f /etc/dnf/protected.d/systemd*
            rm -f /etc/yum/protected.d/grub2*
            rm -f /etc/yum/protected.d/systemd*

            echo "Remove $line: " | tee -a $result
            dnf remove $line -y > ${1}_${arch}_single_remove_info 2>&1 
            if [ $? -ne 0 ]; then
                grep -q 'protected packages' ${1}_${arch}_single_remove_info
                if [ $? -ne 0 ]; then
                    echo "\`removed\` $line \`failed\`." | tee -a $result
                    cat ${1}_${arch}_single_remove_info | tee -a $result
                    success=0
                else
                    echo "\`removed\` $line \`success\`." | tee -a $result
                fi
            else
                grep -q "scriptlet failed" ${1}_${arch}_single_remove_info
                if [ $? -eq 0 ]; then
                    echo "\`removed\` $line \`success\` with \`scriptlet failed\`." | tee -a $result
                    err_info=`grep "scriptlet failed" ${1}_${arch}_single_remove_info`
                    echo "${err_info}" | tee -a $result
                else
                    echo "\`removed\` $line \`success\`." | tee -a $result
                    grep -q "warning:" ${1}_${arch}_single_remove_info
                    if [ $? -eq 0 ]; then
                        echo "with some \`warnings: \`" | tee -a $result
                        grep "warning:" ${1}_${arch}_single_remove_info | tee -a $result
                    fi
                fi
            fi
            dnf autoremove -y
            echo "===============================\n" | tee -a $result
        fi
        echo "dnf install --best $line -y" | tee -a ${1}_${arch}_all_install_info.log
        cat ${1}_${arch}_single_install_info | tee -a ${1}_${arch}_all_install_info.log 

        if [ -f single_remove_info ]; then
            echo "dnf remove $line -y" | tee -a ${1}_${arch}_all_install_info.log
            cat ${1}_${arch}_single_remove_info | tee -a ${1}_${arch}_all_install_info.log
        fi


        if [ $success -ne 1 ]; then
            echo "check ${1} executable failed"
        fi

    done < ${1}_pkg_list

    ls /data/repo | grep -q rpm

    if [ $? -ne 0 ]; then
        echo "no $arch rpm need to install"
    fi
    echo "\n========= **End $arch $1 install.** ========="

    set -e

}

cd $workdir
echo "========= **refresh repo** ========="
if [[ x$tag_name == x"new" ]]; then
    cp OpenCloudOS-Stream-latest.repo /etc/yum.repos.d/OpenCloudOS-Stream.repo
else
    cp OpenCloudOS-Stream-latest.repo /etc/yum.repos.d/OpenCloudOS-Stream-latest.repo
fi
ls /etc/yum.repos.d/

echo "================================"

echo "=========  prepare packages  ==========="

# 1. we install the package
cp $rpmdir/* /data/repo
rm -f /data/repo/*debuginfo*
rm -f /data/repo/*debugsource*
rm -f /data/repo/*.src.rpm

echo "=========  prepare packages done ==========="

createrepo -d /data/repo

# got all the tested rpms
cd /data/repo
if [ -z "$package_name" ]; then
    ls ./*.rpm | xargs -I {} rpm -q --queryformat "%{NAME}-%{VERSION}-%{RELEASE}\n" {} 2>/dev/null > $workdir/${tag_name}_pkg_list
else
    rpm -q --queryformat "%{NAME}-%{VERSION}-%{RELEASE}\n" $package_name 2>/dev/null > $workdir/${tag_name}_pkg_list
fi

cd $workdir
dnf makecache
dnf update -y

check_executable ${tag_name}

if [ $success -ne 1 ]; then
    exit 1
fi
# ==================================================================
