import json
import subprocess
import re

FILTERED_DEPENDENCIES = ['python3']

def extract_package_name(dep):
    match = re.match(r'.*python3(?:\.\d+)?dist\(([^)]+)\).*', dep)
    if match:
        return match.group(1)
    return dep.split(' ')[0]

def get_package_dependencies(package_name):

    package_name = get_rpmname(package_name)

    #print(package_name)
    if package_name == "ERROR":
        return []

    try:
        command = f'rpm-dep -i {package_name} -q'
        subprocess.run(command.split())
        parse_cmd = f"jq -r '.next[] | .pkg_name' dep_tree__{package_name}__install.json | sort | uniq"
        output = subprocess.getoutput(parse_cmd)
        dependencies = output.strip().split('\n')
        return dependencies
    except subprocess.CalledProcessError:
        return []

def get_rpmname(py_name):
    if not py_name.startswith("python-"):
        # try python3dist(ABC)
        command = f"dnf repoquery --whatprovides 'python3dist({py_name})' --latest-limit 1 --queryformat '%{{NAME}}' -q"
        output = subprocess.getoutput(command)

        # try python-ABC
        if output == "":
            # try lower case
            command = f"dnf repoquery --whatprovides 'python3dist({py_name.lower()})' --latest-limit 1 --queryformat '%{{NAME}}' -q"
            output = subprocess.getoutput(command)
            # last chance
            if output == "":
                py_name = f"python3-{py_name}"
                info_command = f'dnf info python3-{py_name}'
                info_result = subprocess.run(info_command.split(), stderr=subprocess.DEVNULL, stdout=subprocess.PIPE, text=True)
                if info_result.returncode != 0:
                    print(f"{py_name} not found in repo!")
                    py_name = "ERROR"
        else:
            py_name = output
    
    else:
        py_name = py_name[7:]
        info_command = f'dnf info python3-{py_name}'
        info_result = subprocess.run(info_command.split(), stderr=subprocess.DEVNULL, stdout=subprocess.PIPE, text=True)
        if info_result.returncode != 0:
            print(f"python3-{py_name} not found in repo!")
            py_name = "ERROR"
        else:
            py_name = f"python3-{py_name}"

    return py_name

def is_in_abnormal_pkg(package_name):
    with open('openstack.sub.list', 'r') as file:
        package_list = file.read().splitlines()

    sub_pkg = get_rpmname(package_name)
    if sub_pkg in package_list:
        print("skip openstack pkg " + sub_pkg)
        return True
    else:
        return False


def check_dependencies(package_data):

    package_name = package_data['package']['key']

    if is_in_abnormal_pkg(package_name):
        return None

    rpm_name = get_rpmname(package_name)
    if rpm_name == "ERROR":
        return None

    local_dependencies = []
    for dep in package_data['dependencies']:
        rpm_name = get_rpmname(dep['key'])
        if rpm_name != "ERROR":
            local_dependencies += [rpm_name]
    repo_dependencies = get_package_dependencies(package_name)

    missing_dependencies = list(set(repo_dependencies) - set(local_dependencies))
    extra_dependencies = list(set(local_dependencies) - set(repo_dependencies))

    #print(local_dependencies)
    #print(repo_dependencies)

    # 过滤 FILTERED_DEPENDENCIES 列表中的依赖项
    missing_dependencies = [dep for dep in missing_dependencies if dep not in FILTERED_DEPENDENCIES]
    extra_dependencies = [dep for dep in extra_dependencies if dep not in FILTERED_DEPENDENCIES]

    #print(missing_dependencies)
    #print(extra_dependencies)
    if not missing_dependencies and not extra_dependencies:
        return None

    return {
        'package_name': get_rpmname(package_name),
        'missing_dependencies': missing_dependencies,
        'extra_dependencies': extra_dependencies
    }

def main():
    with open('packages.json', 'w') as file:
        subprocess.run(['pipdeptree', '-j'], stdout=file)

    with open('packages.json', 'r') as file:
        packages_data = json.load(file)

    result = []
    for package_data in packages_data:
        package_result = check_dependencies(package_data)
        if package_result:
            result.append(package_result)

    with open('result.json', 'w') as file:
        json.dump(result, file, indent=2)

if __name__ == '__main__':
    main()
