#!/usr/bin/bash

# 变量
db_name="ocs23.sqlite"

# 创建一系列依赖关系表
sqlite3 "$db_name" "CREATE TABLE requires (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE conflicts (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE enhances (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE obsoletes (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE provides (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE supplements (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE recommends (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);
                    CREATE TABLE suggests (pkgKey INTEGER, name TEXT, flags TEXT, version TEXT, release TEXT);"


# 将依赖关系插入到相应的表中的函数
insert_dependency() {
    local table_name="$1"
    local pkgKey="$2"
    local name="$3"
    local flags="$4"
    local version="$5"
    local release="$6"

    sqlite3 "$db_name" "INSERT INTO $table_name (pkgKey, name, flags, version, release) \
                       VALUES ('$pkgKey', '$name', '$flags', '$version', '$release');"
}

# 提取数据库中的所有包名和 pkgKey
packages_info=$(sqlite3 "$db_name" "SELECT pkgKey, pkgname FROM packages;")

# 遍历所有包
while IFS='|' read -ra row; do
    pkgKey="${row[0]}"
    pkg_name="${row[1]}"

    # 查询包的各种依赖关系
    for dependency_type in requires conflicts enhances obsoletes provides supplements recommends suggests; do
        # 需要使用 --queryformat "%{NAME} %{VERSION} %{RELEASE}\n" 选项以确保我们有名称、版本和发布号
        rpm_dependencies=$(rpm -q --queryformat "%{NAME} %{VERSION} %{RELEASE}\n" --$dependency_type "$pkg_name")

        # 分析依赖项
        echo "$rpm_dependencies" | while read -r dependency; do
        match=$(echo "$dependency" | grep -oP '^(\S+)(\s+([<>=]+)\s+(\S+))?' | grep -v -E "^rpmlib(")

            if [[ -n $match ]]; then
                name=$(echo "$match" | awk '{print $1}')
                flags=$(echo "$match" | awk '{print $2}')
                version_release=$(echo "$match" | awk '{print $3}')

                if [[ -n $version_release ]]; then
                    version=$(echo "$version_release" | awk -F '-' '{print $1}')
                    release=$(echo "$version_release" | awk -F '-' '{print $2}')
                else
                    version=""
                    release=""
                fi

                # 根据标志转换为对应的版本要求
                case "$flags" in
                    ">") flags="GT" ;;
                    ">=") flags="GE" ;;
                    "=") flags="EQ" ;;
                    "<=") flags="LE" ;;
                    "<") flags="LT" ;;
                    *) flags="" ;;
                esac

                # 将依赖关系插入到相应的表中
                insert_dependency "$dependency_type" "$pkgKey" "$name" "$flags" "$version" "$release"
            fi
        done
    done

done <<<"$packages_info"
