#!/usr/bin/bash

name=`git config --global user.name`

spec_file="$1"
#version=`grep "^Version" ${spec_file} | awk -F":" '{print $2}' | sed s/[[:space:]]//g`
version=`rpmspec -q --queryformat="%{VERSION}\n" ${spec_file} | head -1`
date=`date "+%a %b %d %Y"`

sed -i 's/$RPM_BUILD_ROOT/%{buildroot}/g' ${spec_file}
sed -i 's/${RPM_BUILD_ROOT}/%{buildroot}/g' ${spec_file}
sed -i 's/Source:/Source0:/g' ${spec_file}

#sed perl(:MODULE_COMPAT_%(
sed -i 's/perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))/perl-libs/g' ${spec_file}

# sed tab
sed -i 's/\t/    /g' ${spec_file}

# remove something we dont need
sed -i 's/%{_isa}//g' ${spec_file}
sed -i 's/%{?isa}//g' ${spec_file}
sed -i 's/%{?_isa}//g' ${spec_file}
sed -i 's/%{epoch}://g' ${spec_file}
sed -i 's/%{?epoch:}//g' ${spec_file}
sed -i '/^Epoch/d' ${spec_file}
sed -i '/^Conflicts:/d' ${spec_file}
sed -i '/^Obsoletes/d' ${spec_file}
sed -i '/^%ldconfig_scriptlets/d' ${spec_file}
sed -i '/^%pyproject_check_import/d' ${spec_file}
sed -i '/%python_provide/d' ${spec_file}

# modify release
sed -i '/^Release*/c\Release: 1%{?dist}' ${spec_file}

# 创建子包的 SPEC 文件条目
create_subpkg_entry() {
    local subpkg_name=$1
    local main_pkg_name=$2
    local extra_name=$3
    local files_option=$4
    local files_list=$5

    cat <<EOF
%package -n $subpkg_name
Summary: Subpackage for $main_pkg_name with extra $extra_name dependencies
Requires: $main_pkg_name

%description -n $subpkg_name
This is the subpackage for $main_pkg_name with extra $extra_name dependencies.

%files -n $subpkg_name $files_option
$files_list
EOF
}

temp_file=$(mktemp)

# 读取文件并处理每一行
while IFS= read -r line; do
    if echo "$line" | grep -q "%pyproject_extras_subpkg"; then
        # 初始化变量
        main_pkg_name=""
        extra_names=""
        files_option="-f %{_pyproject_ghost_distinfo}"
        files_list=""

        # 提取参数部分
        params=$(echo "$line" | sed 's/%pyproject_extras_subpkg //')

        # 按空格分割参数并逐个处理
        set -- $params
        while [[ $# -gt 0 ]]; do
            case "$1" in
                -n)
                    main_pkg_name=$2
                    shift 2
                    ;;
                -f)
                    files_option="-f $2"
                    shift 2
                    ;;
                -i)
                    files_list="%ghost $2"
                    shift 2
                    ;;
                *)
                    extra_names=$1
                    shift
                    ;;
            esac
        done

        # Split extra_names by comma and create a subpackage for each one
        IFS=',' read -ra EXTRA_NAMES <<< "$extra_names"
        for extra_name in "${EXTRA_NAMES[@]}"; do
            subpkg_name="${main_pkg_name}+${extra_name}"

            # 调用函数生成新的内容
            new_content=$(create_subpkg_entry "$subpkg_name" "$main_pkg_name" "$extra_name" "$files_option" "$files_list")

            # 插入新内容到原来那一行的位置
            echo "$new_content" >> "$temp_file"
            echo "" >> "$temp_file"
        done
    else
        echo "$line" >> "$temp_file"
    fi
done < ${spec_file}

mv "$temp_file" ${spec_file}

# change the order of SNVR
summary=`grep 'Summary:' ${spec_file} | head -1`
sed -i '0,/Summary:/{/Summary:/d}' ${spec_file}
sed -i "/Name:/i${summary}" ${spec_file}

# add changelog
sed -i '/%changelog/,+10000d' ${spec_file}
echo -e "%changelog\n* ${date} ${name} <${name}@tencent.com> - ${version}-1\n- initial build" >> ${spec_file}
